/**
 * @file
 * @brief ---
 * @author  Andreas Maier <andreas.maier@durham.ac.uk>
 * @version 2.1.1
 *
 * @section LICENSE
 *
 * Copyright 2014 Andreas Maier
 *
 * This file is part of the QCD library.
 *
 * The QCD library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * The QCD library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the QCD library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include <array>

#if __cplusplus >= 201402L
#define QCD_CONSTEXPR_AFTER_CXX14 constexpr
#else
#define QCD_CONSTEXPR_AFTER_CXX14
#endif

namespace QCD{
  namespace detail{

    template<typename T, size_t Rows, size_t Cols>
    class array_2d{
      std::array<T, Rows*Cols> data;
    public:
      QCD_CONSTEXPR_AFTER_CXX14
      T const & operator()(size_t i, size_t j) const{
	return data[i*Cols+j];
      }

      T & operator()(size_t i, size_t j){
	return data[i*Cols+j];
      }

      constexpr size_t rows() const{
	return Rows;
      }

      constexpr size_t columns() const{
	return Cols;
      }
    };
  }
}

#undef QCD_CONSTEXPR_AFTER_CXX14
