/**
 * @file
 * @brief QCD colour factors
 * @author  Andreas Maier <andreas.maier@durham.ac.uk>
 * @version 2.1.1
 *
 * @section LICENSE
 *
 * Copyright 2014 Andreas Maier
 *
 * This file is part of the QCD library.
 *
 * The QCD library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * The QCD library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the QCD library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

namespace QCD{
  //! Dimension of the fundamental representation
  constexpr int n_c = 3;
  //! Dimension of the fundamental representation
  constexpr int d_r = 3;

  //! Quadratic Casimir in the adjoint representation
  constexpr double c_A = 3.;
  //! Quadratic Casimir in the fundamental representation
  constexpr double c_F = 4./3.;
  //! Trace normalisation
  constexpr double T_f = 1./2.;
  //! Product of two symmetrised traces in the adjoint representation
  /**
   * d_abcd_AA = (d^abcd_A)^2, where d^abcd_A is the
   * trace over the symmetrised product of four generators in the
   * adjoint representation
   */
  constexpr double d_abcd_AA = 135.;
  //! Product of two symmetrised traces in the adjoint and fundamental representation
  /**
   * d_abcd_AF = (d^abcd_A) (d^abcd_F), where d^abcd_A/d^abcd_F is the
   * trace over the symmetrised product of four generators in the
   * adjoint/fundamental representation
   */
  constexpr double d_abcd_AF = 15./2.;
  //! Product of two symmetrised traces in the fundamental representation
  /**
   * d_abcd_FF = (d^abcd_F)^2, where d^abcd_F is the
   * trace over the symmetrised product of four generators in the
   * fundamental representation
   */
  constexpr double d_abcd_FF = 5./12.;
  //! Dimension of the adjoint representation
  constexpr double N_A = 8.;
}
