/**
 * @file
 * @brief Helper functions for decoupling
 * @author  Andreas Maier <andreas.maier@durham.ac.uk>
 * @version 2.1.1
 *
 * @section LICENSE
 *
 * Copyright 2018 Andreas Maier
 *
 * This file is part of the QCD library.
 *
 * The QCD library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * The QCD library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the QCD library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

namespace QCD {

  class threshold;

  //! Add flavour in the running of the strong coupling constant
  /**
   * @param alpha_s  alpha_s in the theory with nf flavours
   *                 at the decoupling scale
   * @param thr      Decoupling threshold
   * @param nf       Number of active flavours before decoupling
   * @param order    Perturbative order of decoupling procedure,
   *                 equal to the number of loops in the decoupling constant
   * @returns        alpha_s in the theory with nf+1 active flavours
   */
  double decouple_alpha_s_up(
      double alpha_s, threshold const & thr, int nf, int order
  );

  //! Remove flavour in the running of the strong coupling constant
  /**
   * @param alpha_s  alpha_s in the theory with nf flavours
   *                 at the decoupling scale
   * @param thr      Decoupling threshold
   * @param nf       Number of active flavours before decoupling
   * @param order    Perturbative order of decoupling procedure,
   *                 equal to the number of loops in the decoupling constant
   * @returns        alpha_s in the theory with nf-1 active flavours
   */
  double decouple_alpha_s_down(
      double alpha_s, threshold const & thr, int nf, int order
  );
  
}
