/**
 * @file
 * @brief ---
 * @author  Andreas Maier <andreas.maier@durham.ac.uk>
 * @version 2.1.1
 *
 * @section LICENSE
 *
 * Copyright 2018, 2020 Andreas Maier
 *
 * This file is part of the QCD library.
 *
 * The QCD library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * The QCD library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the QCD library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 */
#include "QCD/decoupling.hpp"

#include <cmath>
#include <stdexcept>

#include "QCD/scheme.hpp"
#include "QCD/threshold.hpp"
#include "QCD/zeta_alpha_s.hpp"

namespace QCD {
  using std::log;

  double decouple_alpha_s_up(
      double alpha_s, threshold const & thr, int nf, int order
  ){
    switch(thr.dec_scheme){
    case scheme::pole:
      return alpha_s*zeta_alpha_s_inv<scheme::pole>(
	  order, nf, 2*log(thr.mu/thr.m), alpha_s
      );
    case scheme::MSbar:
      return alpha_s*zeta_alpha_s_inv<scheme::MSbar>(
	  order, nf, 2*log(thr.mu/thr.m), alpha_s
      );
    case scheme::scale_invariant:
      return alpha_s*zeta_alpha_s_inv<scheme::scale_invariant>(
	  order, nf, 2*log(thr.mu/thr.m), alpha_s
      );
    default:
      throw std::invalid_argument("decoupling not implemented for scheme");
    }
  }

  double decouple_alpha_s_down(
      double alpha_s, threshold const & thr, int nf, int order
  ){
    switch(thr.dec_scheme){
    case scheme::pole:
      return alpha_s*zeta_alpha_s<scheme::pole>(
	  order, nf, 2*log(thr.mu/thr.m), alpha_s
      );
    case scheme::MSbar:
      return alpha_s*zeta_alpha_s<scheme::MSbar>(
	  order, nf, 2*log(thr.mu/thr.m), alpha_s
      );
    case scheme::scale_invariant:
      return alpha_s*zeta_alpha_s<scheme::scale_invariant>(
	  order, nf, 2*log(thr.mu/thr.m), alpha_s
      );
    default:
      throw std::invalid_argument("decoupling not implemented for scheme");
    }
  }

}
