/* test running and decoupling of alpha_s */
#ifdef NDEBUG
#undef NDEBUG
#endif

#include <iostream>
#include <cassert>
#include <array>
#include <cmath>

#include "alpha_s.hpp"
#include "masses.hpp"

#define TST(name, result)                              \
  std::cout << #name << ": ";                        \
  diff = std::abs(name - result);  \
  std::cout << diff;			 \
  assert(diff < ep);			 \
  std::cout << " (ok)" << std::endl;

static constexpr double ep = 1e-6;

int main(){
  double diff;
  using QCD::scheme;

  std::cout << "target precision: " << ep << "\n";
  std::cout << "---------- highest order -----------\n";

  TST(QCD::alpha_s_nf5(150), 0.11009989781359962311);
  TST(QCD::alpha_s_nf5(10), 0.17916943821783215910);

  auto alpha_s_nf6 = QCD::alpha_s_nf5.decouple_up({QCD::mt_OS, QCD::mt_OS});
  TST(alpha_s_nf6(175.000000000001), 0.10782666678927582472);

  auto alpha_s_nf4 = QCD::alpha_s_nf5.decouple_down({2*QCD::mb_OS, QCD::mb_OS});
  TST(alpha_s_nf4(5), 0.21334916179062167917);

  std::cout << "----------- lowest order -----------\n";

  auto alpha_s = QCD::strong_coupling(
      {},
      0
  );

  TST(alpha_s(150), 0.11045628771775595233);
  TST(alpha_s(10), 0.17394253630832081851);
  TST(alpha_s(175.000000000001), 0.10820815172148198989);
  alpha_s_nf4 = alpha_s.decouple_down({2*QCD::mb_OS, QCD::mb_OS});
  TST(alpha_s_nf4(5), 0.20677069643710755627);

  std::cout << "--------- MSbar decoupling ---------\n";

  static constexpr double mb_10 = 3.610;

  auto const MS_dec = QCD::threshold{
    10, mb_10, scheme::MSbar
  };

  alpha_s = QCD::alpha_s_nf5.decouple_down(MS_dec);

  TST(alpha_s(5), 0.21339618753553210933);

  alpha_s = QCD::strong_coupling{
    {alpha_s(5), 5, 4},
    alpha_s.max_running_order
  }.decouple_up(MS_dec);

  diff = std::abs(alpha_s(QCD::mz) - QCD::alpha_s_mz);
  std::cout << "alpha_s(mz): " << diff << '\n';
  assert(diff < (10*ep)); // this is approximately RunDec precision

  std::cout << "---- scale invariant decoupling ----\n";

  static constexpr double mb_mb = 4.163;

  auto const SI_dec = QCD::threshold{
    2*mb_mb, mb_mb, scheme::SI
  };

  alpha_s = QCD::alpha_s_nf5.decouple_down(SI_dec);

  TST(alpha_s(5), 0.21342879414649341406);

  alpha_s = QCD::strong_coupling(
      {alpha_s(5), 5, 4},
      alpha_s.max_running_order
  ).decouple_up(SI_dec);
  diff = std::abs(alpha_s(QCD::mz) - QCD::alpha_s_mz);
  std::cout << "alpha_s(mz): " << diff << '\n';
  assert(diff < (100*ep)); // this is approximately RunDec precision
}
