/* test running of MSbar quark mass */
#ifdef NDEBUG
#undef NDEBUG
#endif

#include <iostream>
#include <iomanip>
#include <cassert>
#include <array>
#include <cmath>

#include "QCD/MSbar_mass.hpp"
#include "QCD/alpha_s.hpp"

#define TST(name, result)                              \
  std::cout << #name << ": ";                        \
  diff = std::abs(name - result);  \
  std::cout << diff;			 \
  assert(diff < ep);			 \
  std::cout << " (ok)" << std::endl;

static constexpr double ep = 1e-5;

int main(){
  using QCD::scheme;
  double diff;
  double constexpr mc0 = 0.986;
  double mu0 = 3.;
  std::array<double, QCD::last_scheme + 1> mub, mb, mut, mt,
    results_nf4, results_nf5_low, results_nf5_high, results_nf6;
  //                   OS,       MS,      SI
  mb               = {{4.7,     3.610,  4.163}};
  mub              = {{2*mb[scheme::OS], 10.,  2*mb[scheme::SI]}};
  mt               = {{173., 162.481, 163.253}};
  mut              = {{mt[scheme::OS], mt[scheme::OS], mt[scheme::SI]}};
  results_nf4      = {{0.8422872437640654, 0.8422872437640654, 0.8422872437640654}};
  results_nf5_low  = {{0.798566195369908, 0.7984677176106406, 0.798494249101116}};
  results_nf5_high = {{0.6027364081042074, 0.6026533324964469, 0.6026995653563516}};
  results_nf6      = {{0.5940279202471904, 0.5939500487967547, 0.5939971787985556}};
  auto alpha_s_nf4 = QCD::strong_coupling{
    QCD::reference_coupling{0.1184, 91.18, 5}, 4
  }.remove({mub[scheme::OS], mb[scheme::OS], scheme::OS});
  auto mc_nf4 = QCD::MSbar_mass(
      {mc0, mu0, 4}, std::move(alpha_s_nf4), 4
  );
  TST(mc_nf4.to_SI(ep), 1.27601951);
  for(
      QCD::scheme s = QCD::first_scheme;
      s <= QCD::last_scheme;
      s = static_cast<QCD::scheme>(s + 1)
  ){
    auto mc_nf5 = mc_nf4.decouple_up({mub[s], mb[s], s});
    auto mc_nf6 = mc_nf5.decouple_up({mut[s], mt[s], s});
    std::cout << "------------------------------ " << s
              << " ------------------------------\n";
    TST(mc_nf4(7), results_nf4[s]);
    TST(mc_nf5(10), results_nf5_low[s]);
    TST(mc_nf5(150), results_nf5_high[s]);
    TST(mc_nf6(180), results_nf6[s]);
  }
}
