/** test zeta_alpha_s
 * reference values from hep-ph/0512060
 * note conventions
 *  zeta_alpha_s: nf = nl + 1
 *  zeta_alpha_s_inv: nf = nl
 **/
#ifdef NDEBUG
#undef NDEBUG
#endif

#include "zeta_alpha_s.hpp"

#include <iostream>
#include <cmath>
#include <cassert>

template<QCD::scheme S>
double diff(unsigned order, unsigned nfpow, double ref){
  if(ref == 0.) return QCD::zeta_alpha_s_nf_coeff<S>(order, nfpow, 0.);
  return (QCD::zeta_alpha_s_nf_coeff<S>(order, nfpow, 0.) - ref)/ref;
}

template<QCD::scheme S>
double diff_inv(unsigned order, unsigned nfpow, double ref){
  if(ref == 0.) return QCD::zeta_alpha_s_inv_nf_coeff<S>(order, nfpow, 0.);
  return (QCD::zeta_alpha_s_inv_nf_coeff<S>(order, nfpow, 0.) - ref)/ref;
}

bool possible_zero(double d){
  static constexpr double ep = 1e-5;
  return std::abs(d) < ep;
}

int main(){
  using QCD::scheme;
  assert(possible_zero(diff<scheme::OS>(0,0,1.)));
  assert(possible_zero(diff<scheme::OS>(1,0,0.)));
  assert(possible_zero(diff<scheme::OS>(2,0,- 0.291667)));
  assert(possible_zero(diff<scheme::OS>(3,0,-5.58636)));
  assert(possible_zero(diff<scheme::OS>(3,1,+ 0.262471)));
  assert(possible_zero(diff<scheme::OS>(4,0,-95.8068)));
  assert(possible_zero(diff<scheme::OS>(4,1,10.1714)));
  assert(possible_zero(diff<scheme::OS>(4,2,-0.239542)));
  assert(possible_zero(diff_inv<scheme::OS>(0,0,1.)));
  assert(possible_zero(diff_inv<scheme::OS>(1,0,0.)));
  assert(possible_zero(diff_inv<scheme::OS>(2,0,0.291667)));
  assert(possible_zero(diff_inv<scheme::OS>(3,0,5.32389)));
  assert(possible_zero(diff_inv<scheme::OS>(3,1,-0.262471)));
  assert(possible_zero(diff_inv<scheme::OS>(4,0,86.1302)));
  assert(possible_zero(diff_inv<scheme::OS>(4,1,-9.69229)));
  assert(possible_zero(diff_inv<scheme::OS>(4,2,0.239542)));

  assert(possible_zero(diff<scheme::MS>(0,0,1.)));
  assert(possible_zero(diff<scheme::MS>(1,0,0.)));
  assert(possible_zero(diff<scheme::MS>(2,0,0.152778)));
  assert(possible_zero(diff<scheme::MS>(3,0,1.05671)));
  assert(possible_zero(diff<scheme::MS>(3,1,-0.0846515)));
  assert(possible_zero(diff<scheme::MS>(4,0,6.1583)));
  assert(possible_zero(diff<scheme::MS>(4,1,-0.965973)));
  assert(possible_zero(diff<scheme::MS>(4,2,-0.0219784)));
  assert(possible_zero(diff_inv<scheme::MS>(0,0,1.)));
  assert(possible_zero(diff_inv<scheme::MS>(1,0,0.)));
  assert(possible_zero(diff_inv<scheme::MS>(2,0,-0.152778)));
  assert(possible_zero(diff_inv<scheme::MS>(3,0,-0.972057)));
  assert(possible_zero(diff_inv<scheme::MS>(3,1,0.0846515)));
  assert(possible_zero(diff_inv<scheme::MS>(4,0,-5.10032)));
  assert(possible_zero(diff_inv<scheme::MS>(4,1,1.00993)));
  assert(possible_zero(diff_inv<scheme::MS>(4,2,0.0219784)));

}
