Needs["CCompilerDriver`"];

prefix = Last[GetEnvironment["LIBQQBAR_THRESHOLD_INSTALL_PREFIX"]];
If[prefix == None,
   Print["Internal error: no prefix"];
   Abort[]
];

version = Last[GetEnvironment["LIBQQBAR_THRESHOLD_VERSION"]];
(* I don't know any way to tell Mathematica to create a versioned library, *)
(* so just add the version number to the library name. *)
(* Apparently '.' is not allowed in the name, so replace it by an underscore *)
strippedVersion = StringReplace[version, "." -> "_"];
libName = "libQQbar_threshold_mathematica_" <> strippedVersion;

Print["Compiling library"]
dir = FileNameJoin[{"bindings", "Mathematica"}];

code = {
   FileNameJoin[{dir, "bindings.cpp"}]
};

CreateLibrary[code, libName,
   "Debug" -> False,
   "Libraries"	-> {
      "QCD", "gsl", "gslcblas", "QQbar_threshold_" <> strippedVersion
   },
   "Defines" -> "NDEBUG",
   "CleanIntermediate" -> True,
   "Language" -> "C++",
   "CompileOptions" -> "-std=c++11 -Wall -Wextra -Wl,-rpath," <>
   FileNameJoin[{prefix, "lib"}],
   (* TODO: this does not work on OS X for unknown reasons *)
   (* "LinkerOptions"-> {"-rpath " <>FileNameJoin[{prefix, "lib"}]}, *)
   "IncludeDirectories" -> {FileNameJoin[{prefix, "include"}]},
   "LibraryDirectories" -> {FileNameJoin[{prefix, "lib"}]}
];

If[$OperatingSystem == "MacOSX",
   Print["Fixing installation paths"];
   Module[
      {wrapperLib, fixPath},
      wrapperLib = FindLibrary[libName];
      fixPath[libname_String] := {
         "install_name_tool",
         "-change", libname <> ".dylib", FileNameJoin[
            {prefix, "lib", libname <> ".dylib"}
         ],
         wrapperLib
      };
      Print[RunProcess[fixPath[#]]] & /@ {
         "libQCD", "libQQbar_threshold"
      };
   ];
];

Print["Installing Mathematica package"];
With[
   {package = "QQbarThreshold.m"},
   source = FileNameJoin[{dir, package <> ".out"}];
   target = FileNameJoin[{$UserBaseDirectory, "Applications", package}];
   Quiet[DeleteFile[target]];
   CopyFile[source, target];
   target = StringReplacePart[target, version, {-2, -3}];
   Quiet[DeleteFile[target]];
   CopyFile[source, target];
   DeleteFile[source];
];
