/**
 * @file
 * @brief Create auxiliary link to main library for linking Mathematica bindings
 *
 * @author  Andreas Maier <andreas.maier@durham.ac.uk>
 * @version 2.2.1
 *
 * @section LICENSE
 *
 * Copyright 2018 Andreas Maier
 *
 * This file is part of the QQbar_threshold library.
 *
 * The QQbar_threshold library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * The QQbar_threshold library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the QQbar_threshold library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 */
#include <unistd.h>
#include <cstdlib>
#include <cstdio>
#include <errno.h>
#include <iostream>
#include <string>
#include <algorithm>
#include <stdexcept>

namespace{
  const std::string lib = "QQbar_threshold";
}

void delete_last_substring(std::string & str, std::string const & substring){
  const auto pos = str.rfind(substring);
  if(pos == std::string::npos){
    throw std::domain_error{
      "String " +  substring + " not found in " + str
    };
  }
  str.erase(pos, substring.length());
}

int main(int argc, char** argv){
  if(argc != 3){
    std::cerr << "Usage: create_aux_link ORIGLINK VERSION\n";
    return EXIT_FAILURE;
  }
  std::string target{argv[1]};
  std::string version{argv[2]};

  delete_last_substring(target, '.' + version);

  const auto lib_pos = target.rfind(lib) + lib.length();
  if(lib_pos == std::string::npos){
    std::cerr << "Library name " << lib
              << " not found in " << target << '\n';
    return EXIT_FAILURE;
  }
  std::replace(begin(version), end(version), '.', '_');
  target.insert(lib_pos, '_' + version);

  std::cerr << "symlink " << target << " -> " << argv[1] << '\n';
  if(symlink(argv[1], target.c_str()) != 0 && errno != EEXIST){
    std::perror("Failed to create symlink");
    return EXIT_FAILURE;
  }

}
