#ifdef NDEBUG
#undef NDEBUG
#endif

#include <complex>
#include <cassert>
#include <array>
#include <iostream>

#include "QQbar_threshold/load_grid.hpp"
#include "QQbar_threshold/Green_functions.hpp"
#include "QQbar_threshold/mass_schemes.hpp"

#define TSTB(name,i) \
  std::cout << #name << ": "; \
  diff = name - results_bottom[i]; \
  std::cout << diff; \
  assert(std::abs(diff)/std::abs(results_bottom[i]) < ep);      \
  std::cout << " (ok)\n";

#define TSTT(name,i) \
  std::cout << #name << ": "; \
  diff = name - results_top[i]; \
  std::cout << diff; \
  assert(std::abs(diff)/std::abs(results_top[i]) < ep);	\
  std::cout << " (ok)\n";

int main(){

  using namespace QQbar_threshold::pert_order;
  using QQbar_threshold::load_grid;
  using QQbar_threshold::grid_directory;
  using QQbar_threshold::bottom_options;
  using QQbar_threshold::top_options;
  using QQbar_threshold::Pole;
  using QQbar_threshold::detail::Green_function;
  using QQbar_threshold::gen_bottom_parameters;
  using QQbar_threshold::gen_top_parameters;
  auto p = gen_bottom_parameters(
      /* irrelevant */ 5, 5, N3LO,
      bottom_options()
  );
  p.alpha = 0;
  p.e_q = 0; //anything regular will do
  p.Yukawa_t2 = 0; //TODO: physical value
  p.contributions.v_width2 = {0.};
  p.contributions.v_width_kinetic = {0.};
  p.contributions.width_ep = {0.};
  typedef std::complex<double> complex;
  complex lam;
  complex diff;
  constexpr double ep = 1e-7;
  std::cout << "target precision: " << ep << "\n";

  //test for bottom
  std::cout << "--------------------------------------------------"
	    << std::endl;

  p.m = 5;
  p.alpha_s = 0.22;
  p.mu = 5;
  p.mu_w = 5;
  p.nl = 4;
  p.mass_scheme = {Pole, 0., 0.};
  p.contributions.ultrasoft[0] = 0.0;

  lam = {5.185449727756858e-6, 10.370899457398806};
  load_grid(grid_directory() + "bbbar_grid.tsv");
  constexpr auto results_bottom = std::array<complex, 4>{{
      {0.6702014261714551, + 1.8333333328803423},
      {-0.53553769376428584, + 0.00178685131897485},
      {-0.2771696541616252, + 1.28990278284541571},
      {-2.4540260461574647799, 0.81068046178628905274}
    }};

  TSTB(Green_function<0>(lam, p), 0)
  TSTB(Green_function<1>(lam, p), 1)
  TSTB(Green_function<2>(lam, p), 2)
  TSTB(Green_function<3>(lam, p), 3)

  //test for top
  std::cout << "--------------------------------------------------"
	    << std::endl;
  p = gen_top_parameters(
      /* irrelevant */ 175, 805, N3LO,
      top_options()
  );

  p.m = 175;
  p.alpha_s = 0.14;
  p.mu = 80;
  p.mu_w = p.m;
  p.nl = 5;
  p.mass_scheme = {Pole, 0., 0.};
  p.contributions.ultrasoft[0] = 1.0;
  p.Yukawa_t2 = 0;
  p.alpha = 0;
  p.contributions.v_width2 = {0.};
  p.contributions.v_width_kinetic = {0.};
  p.contributions.width_ep = {0.};
  lam = {0.43390655872432077, + 0.8107618212297686};
  load_grid(grid_directory() + "ttbar_grid.tsv");
  constexpr auto results_top = std::array<complex, 4>{{
      {344.4298753633245, + 1372.548924622806},
      {-256.37759800167396, - 32.74405722419382},
      {-111.36697888397453, + 349.06523313957877},
      {-35.048335288102293816, 157.59364111843677847}
    }};

  TSTT(Green_function<0>(lam, p), 0)
  TSTT(Green_function<1>(lam, p), 1)
  TSTT(Green_function<2>(lam, p), 2)
  TSTT(Green_function<3>(lam, p), 3)

}
