#ifdef NDEBUG
#undef NDEBUG
#endif

#include <complex>
#include <cassert>
#include <array>
#include <iostream>

#include "load_grid.hpp"
#include "Green_functions.hpp"
#include "mass_schemes.hpp"
#include "scheme_conversion.hpp"

#define TSTB(name,i) \
  std::cout << #name << ": "; \
  diff = name - results_bottom[i]; \
  std::cout << diff; \
  assert(std::abs(diff)/std::abs(results_bottom[i]) < ep);      \
  std::cout << " (ok)\n";

#define TSTT(name,i) \
  std::cout << #name << ": "; \
  diff = name - results_top[i]; \
  std::cout << diff; \
  assert(std::abs(diff)/std::abs(results_top[i]) < ep);	\
  std::cout << " (ok)\n";

int main(){
  using namespace QQbar_threshold::pert_order;
  using QQbar_threshold::load_grid;
  using QQbar_threshold::grid_directory;
  using QQbar_threshold::bottom_options;
  using QQbar_threshold::top_options;
  using QQbar_threshold::PS;
  using QQbar_threshold::mu_f_top;
  using QQbar_threshold::mu_f_bottom;
  using QQbar_threshold::detail::Green_function_TM;
  auto p = gen_bottom_parameters(
      /* irrelevant */ 5, 5, N3LO,
      bottom_options()
  );
  p.ml = 0;
  p.alpha = 1/137.03599907430635;
  typedef std::complex<double> complex;
  complex lam;
  complex diff;
  constexpr double ep = 1e-3;
  std::cout << "target precision: " << ep << "\n";

  //test for bottom
  std::cout << "--------------------------------------------------"
	    << std::endl;

  p.m = 5;
  p.alpha_s = 0.22;
  p.mu = 5;
  p.nl = 4;
  p.mass_scheme = {PS, mu_f_bottom, mu_f_bottom};

  lam = {5.185449727756858e-6, 10.370899457398806};
  load_grid(grid_directory() + "bbbar_grid.tsv");
  constexpr auto results_bottom = std::array<complex, 6>{{
      {0.11204094035838244, + 2.1076970825930122e-10},
      {0.06010759796455368, - 1.3577227114140938e-10},
      {0.090187370389676, + 6.224160720309316e-10},
      {-0.038863174067624254, - 0.1030906053085407},
      {0.012816715360322924, - 0.046990840213253726},
      {-0.0334716378153176, - 0.07288360148085153}
    }};

  TSTB((Green_function_TM<0,1>(lam, p)), 0)
  TSTB((Green_function_TM<0,2>(lam, p)), 1)
  TSTB((Green_function_TM<0,3>(lam, p)), 2)
  TSTB((Green_function_TM<1,1>(lam, p)), 3)
  TSTB((Green_function_TM<1,2>(lam, p)), 4)
  TSTB((Green_function_TM<2,1>(lam, p)), 5)


  //test for top
  std::cout << "--------------------------------------------------"
	    << std::endl;

  p = gen_top_parameters(
      /* irrelevant */ 175, 80, N3LO,
      top_options()
  );
  p.m = 175;
  p.alpha_s = 0.14;
  p.mu = 80;
  p.nl = 5;
  p.mass_scheme = {PS, mu_f_top, mu_f_top};
  lam = {0.43390655872432077, + 0.8107618212297686};
  load_grid(grid_directory() + "ttbar_grid.tsv");
  constexpr auto results_top = std::array<complex, 6>{{
      {55.80934625301535, - 3.520047273544062},
      {40.014969040299015, + 9.458365520265081},
      {36.591846218326786, + 17.761262158941037},
      {-53.121167533626874, - 3.3959427264026707},
      {-55.34486601949448, - 12.896983530740462},
      {19.320835302413787, - 25.055200359508337}
    }};

  TSTT((Green_function_TM<0,1>(lam, p)), 0)
  TSTT((Green_function_TM<0,2>(lam, p)), 1)
  TSTT((Green_function_TM<0,3>(lam, p)), 2)
  TSTT((Green_function_TM<1,1>(lam, p)), 3)
  TSTT((Green_function_TM<1,2>(lam, p)), 4)
  TSTT((Green_function_TM<2,1>(lam, p)), 5)

}
