#ifdef NDEBUG
#undef NDEBUG
#endif

#include <complex>
#include <cassert>
#include <array>
#include <iostream>

#include "load_grid.hpp"
#include "xsection.hpp"
#include "mass_schemes.hpp"

namespace QQbar_threshold{
  namespace detail{
    typedef std::complex<double> complex;
    complex Pi_V(complex lam, parameters const & p);
    complex Pi_A(complex lam, parameters const & p);
  }
}

#define TSTB(name,i) \
  std::cout << #name << ": "; \
  diff = name - results_bottom[i]; \
  std::cout << diff; \
  assert((diff == 0.) || (std::abs(diff)/std::abs(results_bottom[i]) < ep)); \
  std::cout << " (ok)\n";

#define TSTT(name,i) \
  std::cout << #name << ": "; \
  diff = name - results_top[i]; \
  std::cout << diff; \
  assert(std::abs(diff)/std::abs(results_top[i]) < ep);	\
  std::cout << " (ok)\n";


int main(){
  using namespace QQbar_threshold::pert_order;
  using QQbar_threshold::load_grid;
  using QQbar_threshold::grid_directory;
  using QQbar_threshold::bottom_options;
  using QQbar_threshold::top_options;
  using QQbar_threshold::Pole;
  using QQbar_threshold::detail::Pi_V;
  using QQbar_threshold::detail::Pi_A;
  typedef std::complex<double> complex;
  complex lam;
  complex diff;
  constexpr double ep = 1e-4;
  std::cout << "target precision: " << ep << "\n";

  auto p = gen_bottom_parameters(
      /* irrelevant */ 5, 5, N3LO,
      bottom_options()
  );
  p.mass_scheme = {Pole, 0, 0.};//TODO: PS
  p.resum_poles = false;//TODO: true
  p.expand_s = true;
  p.alpha = 0;
  p.e_q = 0; //anything regular will do
  p.Yukawa_t2 = 0; //TODO: physical value
  p.width = 1e-9;

  //test for bottom
  std::cout << "--------------------------------------------------"
	    << std::endl;

  p.m = 5;
  p.alpha_s = 0.22;
  p.mu = 5;
  p.mu_w = 5;
  p.nl = 4;
  p.contributions.ultrasoft[0] = 0.0;
  lam = {5.185449727756858e-6, 10.370899457398806};
  p.E_shifted = - p.m*pow(4./3.*p.alpha_s/(2.*lam),2);
  p.s = pow(real(p.E_shifted) + 2*p.m, 2);
  load_grid(grid_directory() + "bbbar_grid.tsv");
    constexpr auto results_bottom = std::array<complex, 7>{{
	{0.040212085570287305, + 0.109999999972820534},
	{-0.006938730529383302, + 0.069024015085322202},
	{0},
        {-0.027104480071570014, + 0.103878262392822},
	{0.0011907158582160290, + 0.0015921481550615886},
        {-0.19733802861125088, + 0.009726942240545898},
	{0.0008323992301772643, + 0.0045242923060630769}
    }};

  p.order = 0;
  std::cout << "LO:\n";
  TSTB(Pi_V(lam, p),0);
    p.order = 1;
  std::cout << "NLO:\n";
  TSTB(Pi_V(lam, p),1);
  TSTB(Pi_A(lam, p),2);
  std::cout << "NNLO:\n";
    p.order = 2;
  TSTB(Pi_V(lam, p),3);
  TSTB(Pi_A(lam, p),4);
  std::cout << "NNNLO:\n";
  p.order = 3;
  TSTB(Pi_V(lam, p),5);
  TSTB(Pi_A(lam, p),6);

  //test for top
  std::cout << "--------------------------------------------------"
	    << std::endl;

  p = gen_top_parameters(
      /* irrelevant */ 175, 805, N3LO,
      top_options()
  );
  p.m = 175;
  p.alpha_s = 0.14;
  p.mu = 80;
  p.mu_w = 80;
  p.nl = 5;
  p.contributions.ultrasoft[0] = 1.0;
  p.Yukawa_t2 = 0;
  p.alpha = 0;
  p.resum_poles = 0;
  p.mass_scheme = {Pole, 0, 0.};
  lam = {0.43390655872432077, + 0.8107618212297686};
  p.E_shifted = - p.m*pow(4./3.*p.alpha_s/(2.*lam),2);
  p.s = pow(real(p.E_shifted) + 2*p.m, 2);
  load_grid(grid_directory() + "ttbar_grid.tsv");
    constexpr auto results_top = std::array<complex, 6>{{
	{0.016870034711673036, + 0.067226886103974162},
	{0.000303240146489961, + 0.049645188639416950},
        {-0.0028356280019713113, + 0.0636451996379502},
	{-0.00003019094640409561, + 0.00079596818564010429},
        {-0.006130496585197206, + 0.053287221339785446},
	{-0.00019073622371138654, + 0.00109131755168031695}
      }};

  p.order = 0;
  std::cout << "LO:\n";
  TSTT(Pi_V(lam, p),0);
    p.order = 1;
  std::cout << "NLO:\n";
  TSTT(Pi_V(lam, p),1);
  std::cout << "NNLO:\n";
    p.order = 2;
  TSTT(Pi_V(lam, p),2);
  TSTT(Pi_A(lam, p),3);
  std::cout << "NNNLO:\n";
  p.order = 3;
  TSTT(Pi_V(lam, p),4);
  TSTT(Pi_A(lam, p),5);


}
