#ifdef NDEBUG
#undef NDEBUG
#endif

#include <complex>
#include <cassert>
#include <array>
#include <iostream>

#include "QQbar_threshold/load_grid.hpp"
#include "QQbar_threshold/xsection.hpp"
#include "QQbar_threshold/mass_schemes.hpp"
#include "QQbar_threshold/scheme_conversion.hpp"

#define TSTB(name,i) \
  std::cout << #name << ": "; \
  diff = name - results_bottom[i]; \
  std::cout << diff; \
  assert(std::abs(diff)/std::abs(results_bottom[i]) < ep);	\
  std::cout << " (ok)\n";

#define TSTT(name,i) \
  std::cout << #name << ": "; \
  diff = name - results_top[i]; \
  std::cout << diff; \
  assert(std::abs(diff)/std::abs(results_top[i]) < ep);	\
  std::cout << " (ok)\n";

int main(){
  using namespace QQbar_threshold::pert_order;
  using QQbar_threshold::load_grid;
  using QQbar_threshold::grid_directory;
  using QQbar_threshold::Pole;
  using QQbar_threshold::PS;
  using QQbar_threshold::PSshift;
  using QQbar_threshold::mu_f_top;
  using QQbar_threshold::mu_f_bottom;
  using QQbar_threshold::top_options;
  using QQbar_threshold::bottom_options;
  using QQbar_threshold::SM;
  auto opt = top_options();
  opt.alpha = 1/137.03599907430635;
  opt.m_Higgs = 125.9;
  opt.ml = 0;
  opt.beyond_QCD = SM::none;
  opt.resonant_only = true;
  opt.contributions.Cv_QED[0] = 0;
  opt.contributions.Ca_QED[0] = 0;
  opt.contributions.Cv_WZ[0] = 0;
  opt.contributions.Ca_WZ[0] = 0;
  opt.contributions.v_width2 = {0.};
  opt.contributions.v_width_kinetic = {0.};
  opt.contributions.width_ep = {0.};

  using QQbar_threshold::bbbar_R_ratio;
  using QQbar_threshold::bbbar_threshold;
  using QQbar_threshold::ttbar_R_ratio;
  double diff;
  //TODO: improve precision to < 10^-3
  double constexpr ep = 2e-3;
  std::cout << "target precision: " << ep << "\n";
  double constexpr mt = 175.;
  double constexpr width = 1.5;
  double constexpr mb = 5.;

  opt.resum_poles = 6;
  opt.expand_s = true;

  std::cout << "--------------------------------------------------\n";
  load_grid(grid_directory() + "ttbar_grid.tsv");
  std::cout << "top, pole scheme:\n";

  opt.mass_scheme = {Pole, 0, 0.};

  auto results_top = std::array<double, 4>{{
      1.0577518306504974,
      0.8626929545022342,
      1.017785755591901,
      0.9361699510584207
    }};

  for(order o = LO; o <= N3LO; ++o){
    TSTT(ttbar_R_ratio(2*mt + 1, 80, {mt, width}, o, opt), o);
  }

  std::cout << "--------------------------------------------------\n";
  std::cout << "top, PS scheme:\n";

  opt.mass_scheme = {PS, mu_f_top, 0.};

  results_top = std::array<double, 4>{{
      0.9970781928032358,
      0.9037916474538447,
      1.1331603143780862,
      1.0776616297672328
    }};

  for(order o = LO; o <= N3LO; ++o){
    TSTT(ttbar_R_ratio(2*mt + 1, 80, {mt, width}, o, opt), o);
  }

  std::cout << "--------------------------------------------------\n";
  std::cout << "top, PSshift scheme:\n";

  opt.mass_scheme = {PSshift, mu_f_top, 0.};

  results_top = std::array<double, 4>{{
      1.0003193168493443,
      0.9578287076824571,
      1.1392171736487797,
      1.1056194154523962
    }};

  for(order o = LO; o <= N3LO; ++o){
    TSTT(ttbar_R_ratio(2*mt + 1, 80, {mt, width}, o, opt), o);
  }

  std::cout << "--------------------------------------------------\n";
  std::cout << "top, QED effects:\n";

  opt.beyond_QCD = SM::QED;

  results_top = std::array<double, 4>{{
      1.0003193168493443,
      0.9965228934176524,
      1.169118056185494,
      1.1423884788222196
    }};

  for(order o = LO; o <= N3LO; ++o){
    TSTT(ttbar_R_ratio(2*mt + 1, 80, {mt, width}, o, opt), o);
  }

  std::cout << "--------------------------------------------------\n";
  std::cout << "top, QED + Higgs effects:\n";

  opt.beyond_QCD |= SM::Higgs;

  results_top = std::array<double, 4>{{
      1.0003193168493443,
      0.9965228934176524,
      1.2208150725298075,
      1.2049283537136155
    }};

  for(order o = LO; o <= N3LO; ++o){
    TSTT(ttbar_R_ratio(2*mt + 1, 80, {mt, width}, o, opt), o);
  }


  std::cout << "--------------------------------------------------\n";
  std::cout << "bottom, Pole scheme:\n";
  opt = bottom_options();
  opt.beyond_QCD = SM::none;
  opt.expand_s = true;
  opt.alpha = 1/137.03599907430635;
  opt.alpha_s_mZ = 0.11842395335727579;
  opt.contributions.v_width2 = {0.};
  opt.contributions.v_width_kinetic = {0.};
  opt.contributions.width_ep = {0.};
  load_grid(grid_directory() + "bbbar_grid.tsv");

  opt.mass_scheme = {Pole, 0, 0.};

  auto results_bottom = std::array<double, 4>{{
      0.44656007861084845,
      0.28875633069596557,
      0.42329885161542125,
      0.21843172742911954
    }};

  for(order o = LO; o <= N3LO; ++o){
    TSTB(bbbar_R_ratio(2*mb + 1e-3, mb, mb, o, opt), o);
  }

  std::cout << "--------------------------------------------------\n";
  std::cout << "bottom, PS scheme:\n";

  opt.mass_scheme = {PS, mu_f_bottom, 0.};

  results_bottom = std::array<double, 4>{{
      0.4464988885856347,
      0.2887140256298854,
      0.3986731380945475,
      0.19124764201324565
    }};

  for(order o = LO; o <= N3LO; ++o){
    TSTB(bbbar_R_ratio(bbbar_threshold(mb, mb, o, opt)+1e-3, mb, mb, o, opt), o);
  }


  std::cout << "--------------------------------------------------\n";
  std::cout << "bottom, PSshift scheme:\n";

  opt.mass_scheme = {PSshift, mu_f_bottom, 0.};

  results_bottom = std::array<double, 4>{{
      0.4464988885897379,
      0.28136321724354674,
      0.42225449166386686,
      0.20744544319478397
    }};

  for(order o = LO; o <= N3LO; ++o){
    TSTB(bbbar_R_ratio(bbbar_threshold(mb, mb, o, opt)+1e-3, mb, mb, o, opt), o);
  }
}
