#ifdef NDEBUG
#undef NDEBUG
#endif

#include <iostream>
#include <array>
#include <cassert>

#include "width.hpp"

#define TSTT(name,i) \
  std::cout << #name << ": "; \
  diff = name - results_top[i]; \
  std::cout << diff; \
  assert(std::abs(diff)/std::abs(results_top[i]) < ep);	\
  std::cout << " (ok)\n";


namespace qqt = QQbar_threshold;

int main(){
  double diff;
  double constexpr ep = 1e-6;

  auto opt = qqt::top_options();
  opt.mass_scheme.name = qqt::Pole;
  constexpr double mt_pole = 173.;
  constexpr double mu = 80.;
  std::array<double, 3> results_top = {{
      1.4957086539859459,
      1.3516378107314195,
      1.3544641203765172
    }};
  for(qqt::order o = qqt::LO; o <= qqt::N2LO; ++o){
    TSTT(top_width(mu, mt_pole, o, opt), o);
  }

  constexpr double mt_PS = 171.5;
  //TODO: opt.mass_scheme.name = qqt::PSshift;
  opt.mass_scheme = {qqt::PS, 20., 20.};
  results_top = {{
      1.4507211451230206,
      1.3111068847385088,
      1.3440691683382562
    }};
  for(qqt::order o = qqt::LO; o <= qqt::N2LO; ++o){
    TSTT(top_width(mu, mt_PS, o, opt), o);
  }

  constexpr double width = 1.33;
  assert(qqt::ttbar_width(1, mu, {mt_PS, width}, qqt::NLO) == 2*width);
  constexpr double ref_width_corr = -0.00862589;
  diff = qqt::ttbar_width(1, mu, {mt_PS, width}, qqt::N2LO)
    - 2*width - ref_width_corr;
  std::cout << "ttbar_width: " << diff << ' ';
  assert(std::abs(diff) < 1e-8);
  std::cout << "(ok)\n";

}
