#ifdef NDEBUG
#undef NDEBUG
#endif

#include <cassert>
#include <iostream>
#include <iomanip>

#include "scheme_conversion.hpp"

#define TST(name,i) \
  std::cout << #name << ": "; \
  diff = name - results[i]; \
  std::cout << diff; \
  assert(std::abs(diff)/std::abs(results[i]) < ep);    \
  std::cout << " (ok)\n";

namespace QQt = QQbar_threshold;

double top_MS_mass(
    double mu, double m, QQt::order o,
    QQt::options opt = QQt::top_options()
){
  constexpr double precision = 1e-13;
  const double m_pole = QQt::top_pole_mass(mu, m, o, opt);
  double delta_M;
  double m_MS = m;
  do{
    opt.mass_scheme = {QQt::MSshift, m_MS, 0.};
    delta_M = QQt::top_pole_mass(mu, m_MS, o, opt) - m_pole;
    m_MS -= delta_M;
  } while(std::abs(delta_M) > precision);
  return m_MS;
}

double bottom_MS_mass(
    double mu, double m, QQt::order o,
    QQt::options opt = QQt::bottom_options()
){
  constexpr double precision = 1e-15;
  const double m_pole = QQt::bottom_pole_mass(mu, m, o, opt);
  double delta_M;
  double m_MS = m;
  do{
    opt.mass_scheme = {QQt::MSshift, m_MS, 0.};
    delta_M = QQt::bottom_pole_mass(mu, m_MS, o, opt) - m_pole;
    m_MS -= delta_M;
  } while(std::abs(delta_M) > precision);
  return m_MS;
}

//TODO: set to 1e-10 (find out why that fails on some machines)
constexpr double ep = 1e-5;

int main(){
  using namespace QQbar_threshold::pert_order;
  const double mt_PS = 168.;
  const double mut = 50.;
  const double mb_PS = 4.5;
  const double mub = 4.5;
  double diff;
  std::cout << std::setprecision(20);

  /********************* conversion to pole mass ********************/
  auto results = std::array<double, 4>{{
      169.1060971161844293,
      169.49791759425775695,
      169.69812511146392353,
      169.8269739736231827
    }};
  for(order o = LO; o <= N3LO; ++o){
    TST(QQt::top_pole_mass(mut, mt_PS, o), o);
  }
  results = std::array<double, 4>{{
      4.6875284587658878621,
      4.8064510982468622302,
      4.9189687180230468044,
      5.0528809680927508907
  }};
  for(order o = LO; o <= N3LO; ++o){
    TST(QQt::bottom_pole_mass(mub, mb_PS, o), o);
  }

  /********************* conversion to MSbar mass ********************/

  results = std::array<double, 4>{{
      160.24383493100830833,
      160.01744561477215711,
      160.0015707627845245,
      160.03005630110268953
  }};
  for(order o = LO; o <= N3LO; ++o){
    TST(top_MS_mass(mut, mt_PS, o), o);
  }
  results = std::array<double, 4>{{
      4.285684549594748205,
      4.2107476755172452698,
      4.1803797337154255942,
      4.1783534435346618707
  }};
  for(order o = LO; o <= N3LO; ++o){
    TST(bottom_MS_mass(mub, mb_PS, o), o);
  }

}
